#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qmessagebox.h>
#include <qsocketnotifier.h>
#include "capture.h"
#include "scope.h"
#include "spectrum.h"
#include "kwaveview.h"

KWaveView::KWaveView(QWidget *parent, const char *name) : QVBox(parent, name) {
  
  aboutWidget = new QMessageBox(this);
#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
  PCMname =  DEFAULT_PCMNAME;
  open_seq();
  initSeqNotifier();
#else
  PCMcard = DEFAULT_CARD;
  PCMdevice = DEFAULT_DEVICE;
#endif
  adata = new AudioData(BUFSIZE);
  spectrum = 0;
}

KWaveView::~KWaveView()
{
  delete adata;
}

int KWaveView::go() {

  adata->pcm_capture_handle = open_pcm_capture();
  scope = new Scope(this, "Scope", adata);
  scope->refreshScope();
  adata->doCapture = true;
  capture = new Capture(adata);
  if (!capture->running()) {
    capture->start();
  }
  if (capture->running()) {
    if (scope->scopeScreen->getTriggerMode() == TRIGGERMODE_CONTINUOUS) {
      scope->startScope(true);
    }
  }
  return(0);
}

void KWaveView::displaySpectrum() {

    if (!spectrum) {
      spectrum = new Spectrum(0, "Spectrum", adata);
      spectrum->setCaption("Spectrum");
    }
    spectrum->setGeometry(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT);
    spectrum->show();
    spectrum->raise();
    if (capture->running()) {
      if (spectrum->spectrumScreen->getTriggerMode() == SPECTRUM_TRIGGERMODE_CONTINUOUS) {
        spectrum->startSpectrum(true);
      }
    }
}

void KWaveView::displayAbout() {
 
    aboutWidget->about(this, "About KWaveView", "KWaveView 1.1.4\n"
                     "by Matthias Nagorni\n"
                     "(c)2001 SuSE GmbH Nuremberg\n"
                     "(c)2002 SuSE AG Nuremberg\n\n");
    aboutWidget->raise();
}

int KWaveView::setPeriodsize(int p_periodsize){

  adata->setPeriodsize(p_periodsize);
  return(adata->periodsize);
}

int KWaveView::setPeriods(int p_periods){

  adata->setPeriods(p_periods);
  return(adata->periods);
}

int KWaveView::setRate(int p_rate){

  adata->setRate(p_rate);
  return(adata->rate);
}

int KWaveView::setChannels(int p_channels){

  adata->setChannels(p_channels);
  return(adata->channels);
}

#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
int KWaveView::setPCMname(QString p_name){

  PCMname = p_name;
  return(0);
}

#else

int KWaveView::setPCMcard(int p_card){

  PCMcard = p_card;
  return(PCMcard);
}

int KWaveView::setPCMdevice(int p_device){
  
  PCMdevice = p_device;
  return(PCMdevice);
}

#endif

#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1

void KWaveView::open_seq() {

  int portid;

  if (snd_seq_open(&seq_handle, "hw", SND_SEQ_OPEN_DUPLEX, 0) < 0) {
    fprintf(stderr, "Error opening ALSA sequencer.\n");
    exit(1);
  }
  snd_seq_set_client_name(seq_handle, "KWaveView");
  if ((portid = snd_seq_create_simple_port(seq_handle, "KWaveView",
            SND_SEQ_PORT_CAP_WRITE|SND_SEQ_PORT_CAP_SUBS_WRITE,
            SND_SEQ_PORT_TYPE_APPLICATION)) < 0) {
    fprintf(stderr, "Error creating sequencer port.\n");
    exit(1);
  }
}
 
int KWaveView::initSeqNotifier() {

  int alsaEventFd = 0;

  struct pollfd pfd[1];
  snd_seq_poll_descriptors(seq_handle, pfd, 1, POLLIN);
  alsaEventFd = pfd[0].fd;
  QSocketNotifier *seqNotifier = new QSocketNotifier(alsaEventFd, QSocketNotifier::Read);
  QObject::connect(seqNotifier, SIGNAL(activated(int)),
                   this, SLOT(midiAction(int)));
  return(0);
}

#endif

snd_pcm_t *KWaveView::open_pcm_capture() {
  
  snd_pcm_t *pcm_handle;

#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
  snd_pcm_stream_t stream_capture = SND_PCM_STREAM_CAPTURE;
  snd_pcm_hw_params_t *hwparams;

  if (snd_pcm_open(&pcm_handle, PCMname.latin1(), stream_capture, 0) < 0) {
    fprintf(stderr, "Error opening PCM device %s\n", PCMname.latin1());
    exit(1);
  }
  snd_pcm_hw_params_alloca(&hwparams);
  if (snd_pcm_hw_params_any(pcm_handle, hwparams) < 0) {
    fprintf(stderr, "Can not configure this PCM device.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_access(pcm_handle, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED) < 0) {
    fprintf(stderr, "Error setting access.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_format(pcm_handle, hwparams, SND_PCM_FORMAT_S16_LE) < 0) {
    fprintf(stderr, "Error setting format.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_rate_near(pcm_handle, hwparams, adata->rate, 0) < 0) {
    fprintf(stderr, "Error setting rate.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_channels(pcm_handle, hwparams, adata->channels) < 0) {
    fprintf(stderr, "Error setting channels.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_periods(pcm_handle, hwparams, adata->periods, 0) < 0) {
    fprintf(stderr, "Error setting periods.\n");
    exit(1);
  }
  if (snd_pcm_hw_params_set_buffer_size(pcm_handle, hwparams, adata->frames * adata->periods) < 0) {
    fprintf(stderr, "Error setting buffersize.\n");
    exit(1);
  }
  if (snd_pcm_hw_params(pcm_handle, hwparams) < 0) {
    fprintf(stderr, "Error setting HW params.\n");
    exit(1);
  }

#else
  snd_pcm_channel_params_t hwparams;
    
  if (snd_pcm_open(&pcm_handle, PCMcard, PCMdevice, SND_PCM_OPEN_CAPTURE) < 0) {
    fprintf(stderr, "Error opening PCM card %d device %d\n", PCMcard, PCMdevice);
    exit(1);
  }
  memset(&hwparams, 0, sizeof(hwparams));
  hwparams.format.rate = adata->rate;
  hwparams.format.format = SND_PCM_SFMT_S16_LE;
  hwparams.format.interleave = 1;
  hwparams.format.voices = adata->channels;
  hwparams.mode = SND_PCM_MODE_BLOCK;
  hwparams.channel = SND_PCM_CHANNEL_CAPTURE;
  hwparams.start_mode = SND_PCM_START_DATA;  
  hwparams.stop_mode = SND_PCM_STOP_STOP;    
  hwparams.buf.block.frag_size = adata->periodsize; // in Bytes, NICHT Frames
  hwparams.buf.block.frags_max = adata->periods; // Fragments
  hwparams.buf.block.frags_min = 1;
  if (snd_pcm_channel_params(pcm_handle, &hwparams) < 0) {
    fprintf(stderr, "Can not configure this PCM device.\n");
    exit(1);
  }
  if (snd_pcm_channel_prepare(pcm_handle, SND_PCM_CHANNEL_CAPTURE) < 0) {
    fprintf(stderr, "Can not prepare this PCM device.\n");
    exit(1);
  }
#endif
  return(pcm_handle);
}

#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1   

void KWaveView::midiAction(int fd) {

  snd_seq_event_t *ev;

  do {
    snd_seq_event_input(seq_handle, &ev);
    if ((ev->type == SND_SEQ_EVENT_NOTEON) && (ev->data.note.velocity > 0)) {
      scope->midiShot();
    }
    snd_seq_free_event(ev);
  } while (snd_seq_event_input_pending(seq_handle, 0) > 0);
}

#endif
