#include <stdio.h>      
#include <stdlib.h>     
#include <getopt.h>  
#include <qapplication.h>
#include <qmainwindow.h>
#include <qmenubar.h>   
#include <qpopupmenu.h>
#include <qvbox.h>

#include "kwaveview.h"

static struct option options[] =
        {{"bufsize", 1, 0, 'b'},
         {"frag", 1, 0, 'f'},   
         {"rate", 1, 0, 'r'},   
         {"help", 0, 0, 'h'},
#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
         {"soundcard", 1, 0, 'c'},
#else
         {"soundcard", 1, 0, 'c'},
         {"device", 1, 0, 'd'},   
#endif
         {0, 0, 0, 0}};

int main(int argc, char *argv[])  
{

  QString aboutText ("KWaveView 1.1.4\n"
                     "by Matthias Nagorni\n"
                     "(c)2001 SuSE GmbH Nuremberg\n"
                     "(c)2002 SuSE AG Nuremberg\n\n");
  QApplication *qApp = new QApplication(argc, argv);
  QMainWindow *top = new QMainWindow();
  QVBox *view = new QVBox(top); 
  KWaveView *kwaveview = new KWaveView(view);
  int getopt_return;
  int option_index; 
#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
  while((getopt_return = getopt_long(argc, argv, "hb:f:c:r:", options, &option_index)) >= 0) {
#else
  while((getopt_return = getopt_long(argc, argv, "hb:f:c:d:r:", options, &option_index)) >= 0) {
#endif
    switch(getopt_return) {
    case 'b': 
        kwaveview->setPeriodsize(atoi(optarg));
        break;
    case 'f': 
        kwaveview->setPeriods(atoi(optarg));
        break;
    case 'r': 
        kwaveview->setRate(atoi(optarg));
        break;
    case 'c': 
#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
        kwaveview->setPCMname(optarg);
        break; 
#else
        kwaveview->setPCMcard(atoi(optarg));
        break;
    case 'd': 
        kwaveview->setPCMdevice(atoi(optarg));
        break;
#endif
    case 'h':
        printf("\n%s", aboutText.latin1());
        printf("--bufsize <bytes>            Buffersize [16384]\n");
        printf("--frag <num>                 Number of fragments [2]\n");
        printf("--rate <samples/s>           Samplerate [44100]\n");
#if SND_LIB_MINOR >= 6 || SND_LIB_MAJOR >= 1
        printf("--soundcard <plug>           Soundcard [plughw:0,0]\n\n");
#else
        printf("--soundcard <num>            Soundcard number [0]\n");
        printf("--device <num>               Device number [0]\n\n");
#endif
        exit(EXIT_SUCCESS);
        break;
    }
  }  
  QPopupMenu *filePopup = new QPopupMenu(top);
  QPopupMenu *viewPopup = new QPopupMenu(top);
  QPopupMenu *aboutMenu = new QPopupMenu(top);
  top->menuBar()->insertItem("&File", filePopup);
  top->menuBar()->insertSeparator();
  top->menuBar()->insertItem("&View", viewPopup);
  top->menuBar()->insertSeparator();
  top->menuBar()->insertItem("&About", aboutMenu);
  filePopup->insertItem("&Quit", qApp, SLOT(quit()));
  viewPopup->insertItem("&Spectrum", kwaveview, SLOT(displaySpectrum()));
  aboutMenu->insertItem("About KWaveView", kwaveview, SLOT(displayAbout()));
  kwaveview->go();
  top->setGeometry(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT);
  top->setCentralWidget(view);
  top->show();
  qApp->setMainWidget(top);
  return qApp->exec();
  return (0);
}
