#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qhbuttongroup.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include "audiodata.h"
#include "scope.h"

Scope::Scope(QWidget *parent, const char *name, AudioData *p_adata) : QVBox(parent, name) {

  adata = p_adata;
  scopeScreen = new ScopeScreen(this, "ScopeScreen", adata);
  timer = new QTimer(this);
  QObject::connect(timer, SIGNAL(timeout()),
                   this, SLOT(timerProc()));
  initControls();
}

Scope::~Scope()
{
}


void Scope::refreshScope() {

  scopeScreen->refreshScope();
}

void Scope::initControls() {

  int l1;

  QHBox *scopeControls = new QHBox(this);
  scopeControls->setSpacing(10);
  scopeControls->setMargin(10);
  QHBox *scopeSlider = new QHBox(this);
  scopeSlider->setSpacing(10);    
  scopeSlider->setMargin(10);
  QHBox *triggerGroupBox = new QHBox(this);
  triggerGroupBox->setMargin(10);
  QGroupBox *triggerGroup = new QGroupBox(7, Qt::Horizontal, "Trigger", triggerGroupBox);
  QWidget *dummy1 = new QWidget(scopeControls);
  QHBox *ch1SelectorBox = new QHBox(scopeControls);
  QWidget *dummy2 = new QWidget(scopeControls);
  QHBox *ch2SelectorBox = new QHBox(scopeControls);
  QWidget *dummy3 = new QWidget(scopeControls);
  dummy1->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  dummy2->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  dummy3->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  ch1SelectorBox->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
  ch2SelectorBox->setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum));
  QLabel *ch1Label = new QLabel(ch1SelectorBox);
  ch1Label->setText("Ch1: ");
  ch1Selector = new QComboBox(ch1SelectorBox);
  QLabel *ch2Label = new QLabel(ch2SelectorBox);       
  ch2Label->setText("Ch2: ");     
  ch2Selector = new QComboBox(ch2SelectorBox);
    ch1Selector->insertItem("None");
    ch2Selector->insertItem("None");
  for (l1 = 0; l1 < adata->channels; l1++) {
    ch1Selector->insertItem(QString::number(l1));
    ch2Selector->insertItem(QString::number(l1));
  }
  ch1Selector->setCurrentItem(1);
  ch2Selector->setCurrentItem(2);
  QHButtonGroup *radioButtons = new QHButtonGroup(scopeControls);
  radioButtons->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QWidget *dummy4 = new QWidget(scopeControls);
  dummy4->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  normButton = new QRadioButton("Normal   ", radioButtons);
  sumButton = new QRadioButton("Ch1+Ch2   ", radioButtons);
  diffButton = new QRadioButton("Ch1-Ch2", radioButtons);
  radioButtons->setButton(scopeScreen->getMode());
  QObject::connect(radioButtons, SIGNAL(clicked(int)),
                   this, SLOT(modeChanged(int)));
  QObject::connect(ch1Selector, SIGNAL(highlighted(int)),
                   this, SLOT(ch1Changed(int)));
  QObject::connect(ch2Selector, SIGNAL(highlighted(int)),
                   this, SLOT(ch2Changed(int))); 

  QVBox *timeScaleSliderBox = new QVBox(scopeSlider);
  QHBox *timeScaleLabelBox = new QHBox(timeScaleSliderBox);
  QLabel *timeScaleMinLabel = new QLabel(timeScaleLabelBox);
  timeScaleMinLabel->setText(QString::number(MIN_TIMESCALE)+" ms");
  QWidget *dummy5 = new QWidget(timeScaleLabelBox);
  dummy5->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *timeScaleLabel = new QLabel(timeScaleLabelBox);
  timeScaleLabel->setText("Timescale");
  QWidget *dummy6 = new QWidget(timeScaleLabelBox);
  dummy6->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *timeScaleMaxLabel = new QLabel(timeScaleLabelBox);
  timeScaleMaxLabel->setText(QString::number(MAX_TIMESCALE)+" ms");
  timeScaleSlider = new QSlider(MIN_TIMESCALE, MAX_TIMESCALE, int(MAX_TIMESCALE / 10.0), 
                                scopeScreen->getTimeScale(), QSlider::Horizontal, timeScaleSliderBox, "TimeScale");
  timeScaleSlider->setTickInterval(int(MAX_TIMESCALE / 10.0));
  timeScaleSlider->setTickmarks(QSlider::Below);

  QLabel *dummyLabel1 = new QLabel(scopeSlider);
  dummyLabel1->setText(" ");

  QVBox *zoomSliderBox = new QVBox(scopeSlider);
  QHBox *zoomLabelBox = new QHBox(zoomSliderBox);  
  QLabel *zoomMinLabel = new QLabel(zoomLabelBox); 
  zoomMinLabel->setText(QString::number((double)MIN_ZOOM / 10.0));
  QWidget *dummy7 = new QWidget(zoomLabelBox);
  dummy7->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *zoomLabel = new QLabel(zoomLabelBox);
  zoomLabel->setText("Gain");
  QWidget *dummy8 = new QWidget(zoomLabelBox);
  dummy8->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *zoomMaxLabel = new QLabel(zoomLabelBox); 
  zoomMaxLabel->setText(QString::number((double)MAX_ZOOM / 10.0));
  zoomSlider = new QSlider(MIN_ZOOM, MAX_ZOOM, int(MAX_ZOOM / 10.0), int((double)scopeScreen->getZoom() * 10.0),
                                QSlider::Horizontal, zoomSliderBox, "Gain");          
  zoomSlider->setTickInterval(int(MAX_ZOOM / 10.0));                    
  zoomSlider->setTickmarks(QSlider::Below); 

  QPushButton *zoom1Button = new QPushButton("Gain 1", scopeSlider);
  QObject::connect(zoom1Button, SIGNAL(clicked()),
                   this, SLOT(setZoom1()));

  triggerSlider = new QSlider(adata->min, adata->max, int(adata->max / 5.0), scopeScreen->getTriggerThrs(),
                                QSlider::Horizontal, triggerGroup, "Trigger");          
  triggerSlider->setTickInterval(int(adata->max / 5.0));                    
  triggerSlider->setTickmarks(QSlider::Below); 
  triggerSlider->setMinimumWidth(100);
  QLabel *dummyLabel2 = new QLabel(triggerGroup);
  dummyLabel2->setText("  ");
  QHButtonGroup *edgeButtons = new QHButtonGroup("Edge", triggerGroup);
  risingButton = new QRadioButton("Rising  ", edgeButtons);
  fallingButton = new QRadioButton("Falling  ", edgeButtons);
  QLabel *dummyLabel3 = new QLabel(triggerGroup);
  dummyLabel3->setText("  ");
  edgeButtons->setButton(scopeScreen->getEdge());
  QHButtonGroup *triggerModeButtons = new QHButtonGroup("Mode", triggerGroup);
  continuousButton = new QRadioButton("Continuous  ", triggerModeButtons);
  triggeredButton = new QRadioButton("Triggered  ", triggerModeButtons);
  singleButton = new QRadioButton("Single", triggerModeButtons);
  midiButton = new QRadioButton("MIDI", triggerModeButtons);
  QLabel *dummyLabel4 = new QLabel(triggerGroup);
  dummyLabel4->setText("  ");
  triggerButton = new QPushButton("Trigger", triggerGroup); 
  triggerModeButtons->setButton(scopeScreen->getTriggerMode());

  QObject::connect(edgeButtons, SIGNAL(clicked(int)),
                   this, SLOT(edgeChanged(int)));
  QObject::connect(triggerModeButtons, SIGNAL(clicked(int)),
                   this, SLOT(triggerModeChanged(int)));
  QObject::connect(timeScaleSlider, SIGNAL(valueChanged(int)),
                   this, SLOT(timeScaleChanged(int)));
  QObject::connect(zoomSlider, SIGNAL(valueChanged(int)),
                   this, SLOT(zoomChanged(int)));
  QObject::connect(triggerSlider, SIGNAL(valueChanged(int)),
                   this, SLOT(triggerThrsChanged(int)));
  QObject::connect(triggerButton, SIGNAL(clicked()),
                   scopeScreen, SLOT(singleShot()));
}

void Scope::modeChanged(int id) {

  switch(id) {
  case 0: scopeScreen->setMode(MODE_NORMAL);
          break;
  case 1: scopeScreen->setMode(MODE_SUM);
          break;
  case 2: scopeScreen->setMode(MODE_DIFF);
          break; 
  }  
}

void Scope::triggerModeChanged(int id) {

  if (((scopeScreen->getTriggerMode() == TRIGGERMODE_SINGLE) 
      || (scopeScreen->getTriggerMode() == TRIGGERMODE_MIDI))
      && ((id == TRIGGERMODE_CONTINUOUS) || (id == TRIGGERMODE_TRIGGERED))) {
    scopeScreen->noScroll();
    timer->start(scopeScreen->getTimeScale(), true);
  } 
  switch(id) {
  case 0: scopeScreen->setTriggerMode(TRIGGERMODE_CONTINUOUS);
          break;
  case 1: scopeScreen->setTriggerMode(TRIGGERMODE_TRIGGERED);
          break;
  case 2: scopeScreen->setTriggerMode(TRIGGERMODE_SINGLE);
          break;
  case 3: scopeScreen->setTriggerMode(TRIGGERMODE_MIDI);
          break;
  }  
}

void Scope::edgeChanged(int id) {

  switch(id) {
  case 0: scopeScreen->setEdge(EDGE_RISING);
          break;
  case 1: scopeScreen->setEdge(EDGE_FALLING);
          break;
  }  
}    


void Scope::ch1Changed(int index) {

  scopeScreen->setCh1(index - 1);
}

void Scope::ch2Changed(int index) { 

  scopeScreen->setCh2(index - 1);  
}

void Scope::timeScaleChanged(int val) { 

  scopeScreen->setTimeScale(val);
}

void Scope::zoomChanged(int val) {
     
  scopeScreen->setZoom((double)val / 10.0);
}

void Scope::triggerThrsChanged(int val) {
 
  scopeScreen->setTriggerThrs(val);
}

void Scope::timerProc() {                           

  if ((scopeScreen->getTriggerMode() != TRIGGERMODE_SINGLE) 
    && (scopeScreen->getTriggerMode() != TRIGGERMODE_MIDI)) {
    timer->start(scopeScreen->getTimeScale(), true);
  }
  refreshScope();
}

void Scope::startScope(bool run) {

  timerProc();
}

void Scope::setZoom1() {
 
  zoomSlider->setValue(int((double)scopeScreen->setZoom(1.0) * 10.0));
}

void Scope::midiShot() {

  if (scopeScreen->getTriggerMode() == TRIGGERMODE_MIDI) {
    timer->singleShot((int)(((double)(scopeScreen->getTriggerThrs() - adata->min) 
                       / (double)(adata->max - adata->min) * 0.8 + 0.7) * (double)scopeScreen->getTimeScale()), 
                      scopeScreen, SLOT(singleShot()));
  }  
}
