#ifndef KSCOPE_H
#define KSCOPE_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include "scopescreen.h"

#define SCOPE_BORDER                5
#define CONTROLS_HEIGHT            50
#define MIN_TIMESCALE              10
#define MAX_TIMESCALE            1000
#define MIN_ZOOM                    5
#define MAX_ZOOM                  100

class Scope : public QVBox
{
  Q_OBJECT

  private:
    AudioData *adata;
    QComboBox *ch1Selector;
    QComboBox *ch2Selector;
    QRadioButton *sumButton, *diffButton, *normButton;
    QRadioButton *risingButton, *fallingButton, *continuousButton, *triggeredButton, *singleButton, *midiButton;
    QPushButton *triggerButton;
    QSlider *timeScaleSlider, *zoomSlider, *triggerSlider;
    QTimer *timer;

  public:
    ScopeScreen *scopeScreen;

  private:
    void initControls();

  public:
    Scope(QWidget* parent=0, const char *name=0, AudioData *p_adata=0);
    ~Scope();
    void startScope(bool run);

  public slots:
    void modeChanged(int id);
    void triggerModeChanged(int id);
    void edgeChanged(int id);
    void ch1Changed(int index);
    void ch2Changed(int index);
    void timeScaleChanged(int val);
    void zoomChanged(int val);
    void setZoom1();
    void triggerThrsChanged(int val);
    void refreshScope();
    void timerProc();
    void midiShot();
};

#endif

