#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qpainter.h>
#include <qsizepolicy.h>
#include <qsize.h>
#include <qscrollview.h>
#include <qevent.h>
#include "audiodata.h"
#include "scope.h"

ScopeScreen::ScopeScreen(QWidget* parent, const char *name, AudioData *p_adata) 
                   : QScrollView(parent, name, Qt::WResizeNoErase | Qt::WRepaintNoErase)
{
  adata = p_adata;
  scopebuf = (unsigned char*)malloc(adata->bufsize);
  scopebufValidBytes = 0;
  mode = MODE_NORMAL;
  edge = EDGE_RISING;
  triggerMode = TRIGGERMODE_CONTINUOUS;
  triggerThrs = 0;
  triggered = false;
  dragFlag = false;
  ch1 = 0;
  ch2 = 1;
  zoom = 1;
  timeScale = 100;
  timeScaleLastTrigger = timeScale;
  timeScaleBytes = adata->framesize * (int)((double)adata->rate * (double)timeScale / 1000.0);
  setPalette(QPalette(QColor(0, 80, 0), QColor(0, 80, 0)));
  setVScrollBarMode(QScrollView::AlwaysOff);
  timer = new QTimer(this);
  QObject::connect(timer, SIGNAL(timeout()),
                   this, SLOT(timerProc()));
  QObject::connect(this, SIGNAL(contentsMoving(int, int)), this, SLOT(moveHandler(int, int)));
}

ScopeScreen::~ScopeScreen()
{
  free(scopebuf);
}

void ScopeScreen::viewportPaintEvent(QPaintEvent *) {
  
  int l1, l_lower;
  double xscale, yscale;
  int x1, x2, y1ch1, y1ch2, y2ch1, y2ch2, thrs, vx, vy, vw, vh;
  short s1, s2;
  QPixmap pm(visibleWidth(), visibleHeight());
  QPainter p(&pm);

  /* fix 'may be used uninitialized in this function' */
  y1ch1 = y1ch2 = 0;

  pm.fill(QColor(0, 80, 0));
  vx = 0; vy = 0; vw = viewport()->width(); vh = viewport()->height();
  xscale = (double)vw / ((double)timeScaleBytes / (double)adata->framesize);
  yscale = zoom * (double)contentsHeight() / 65536.0;
  thrs = int(yscale * (double)triggerThrs);
  if (triggerMode == TRIGGERMODE_TRIGGERED) {
    p.setPen(QColor(0, 255, 255));
    p.drawLine(vx, vy + (vh >> 1) + thrs, vx + 10, vy + (vh >> 1) + thrs);
    p.setPen(QColor(0, 220, 0));
  }
  l_lower = adata->framesize * (int)((double)contentsX() / (double)xscale);
  for (l1 = 0; l1 < timeScaleBytes / adata->framesize - 1; l1++) {
    x1 = int((double)l1 * xscale);
    x2 = int((double)(l1+1) * xscale);
    if (ch1 < 0) {
      s1 = 0;
    } else {
      s1 = scopebuf[l_lower + adata->framesize*l1+ch1*adata->samplesize] 
         | (scopebuf[l_lower + adata->framesize*l1+ch1*adata->samplesize+1] << 8);
    }
    if (ch2 < 0) {
      s2 = 0;
    } else {
      s2 = scopebuf[l_lower + adata->framesize*l1+ch2*adata->samplesize] 
         | (scopebuf[l_lower + adata->framesize*l1+ch2*adata->samplesize+1] << 8); 
    }
    switch (mode) {
    case MODE_NORMAL:
      y1ch1 = int(yscale * double(s1));
      y1ch2 = int(yscale * double(s2));
      break;    
    case MODE_SUM:
      y1ch1 = int(yscale * double(s1+s2));
      break;
    case MODE_DIFF:
      y1ch1 = int(yscale * double(s1-s2));
      break;
    }
    if (ch1 < 0) {
      s1 = 0;
    } else { 
      s1 = scopebuf[l_lower + adata->framesize*(l1+1)+ch1*adata->samplesize]
         | (scopebuf[l_lower + adata->framesize*(l1+1)+ch1*adata->samplesize+1] << 8);
    }
    if (ch2 < 0) {
      s2 = 0;
    } else {
      s2 = scopebuf[l_lower + adata->framesize*(l1+1)+ch2*adata->samplesize]
         | (scopebuf[l_lower + adata->framesize*(l1+1)+ch2*adata->samplesize+1] << 8);
    }
    switch (mode) {  
    case MODE_NORMAL:
      y2ch1 = int(yscale * double(s1));
      y2ch2 = int(yscale * double(s2));
      if (ch1 >= 0) {
        p.setPen(QColor(0, 220, 0));
        p.drawLine(vx + x1, (vh >> 1) + y1ch1, vx + x2, (vh >> 1) + y2ch1);
      }
      if (ch2 >= 0) {
        p.setPen(QColor(255, 255, 0));
        p.drawLine(vx + x1, (vh >> 1) + y1ch2, vx + x2, (vh >> 1) + y2ch2);
      }
      break;      
    case MODE_SUM:
      y2ch1 = int(yscale * double(s1+s2));
      if ((ch1 >= 0) && (ch2 >= 0)) {
        p.setPen(QColor(0, 220, 0));
        p.drawLine(vx + x1, (vh >> 1) + y1ch1, vx + x2, (vh >> 1) + y2ch1);
      }
      break;
    case MODE_DIFF:
      y2ch1 = int(yscale * double(s1-s2));
      if ((ch1 >= 0) && (ch2 >= 0)) {
        p.setPen(QColor(0, 220, 0));
        p.drawLine(vx + x1, (vh >> 1) + y1ch1, vx + x2, (vh >> 1) + y2ch1);
      }
      break;
    }
  }
  if (!dragFlag) {
    p.setPen(QColor(0, 220, 0));
    p.drawRect(vx, vy, vx + vw, vy + vh);
    p.drawLine(vx, vy + (vh >> 1), vx + vw - 1, vy + (vh >> 1));
    p.drawLine(vx + (vw >> 1), vx, vx + (vw >> 1), vy + vh);
    p.drawText(vx + 5, vy + 20, QString::number(timeScale)+" ms");
  }
  bitBlt(viewport(), 0, 0, &pm);
}

void ScopeScreen::refreshScope() {

  int l1, ofs;
  short s1, s2;

  adata->readofs = adata->writeofs - adata->periodsize - timeScaleBytes;
  if (adata->readofs < 0 ) {  
    adata->readofs+=adata->bufsize;
  }
  ofs = adata->readofs;
  if ((triggerMode == TRIGGERMODE_TRIGGERED) && (ch1 >=0)) {
    l1 = 0;
    triggered = false;
    while(l1 < TRIGGER_RANGE) {
      s1 = adata->buf[ofs+ch1*adata->samplesize]
         | (adata->buf[ofs+ch1*adata->samplesize+1] << 8);
      ofs -= adata->framesize;
      if (ofs < 0 ) {  
        ofs+=adata->bufsize;
      }
      s2 = adata->buf[ofs+ch1*adata->samplesize]                     
         | (adata->buf[ofs+ch1*adata->samplesize+1] << 8); 
      if (edge == EDGE_RISING) {
        if ((s1 < triggerThrs) && (s2 > triggerThrs)) {
          adata->readofs = ofs;
          triggered = true;
          break;
        }
      } else {
      if ((s1 > triggerThrs) && (s2 < triggerThrs)) {
          adata->readofs = ofs;
          triggered = true;
          break;
        }
      }
      ofs -= adata->framesize;
      if (ofs < 0 ) {
        ofs+=adata->bufsize;  
      }
      l1++;
    }
  }
  if ((triggerMode != TRIGGERMODE_TRIGGERED) || triggered) {
    for (l1 = 0; l1 < timeScaleBytes; l1++) {
      scopebuf[l1] = adata->buf[ofs++];
      if (ofs >= adata->bufsize) {
        ofs -= adata->bufsize;
      }
    }
    scopebufValidBytes = timeScaleBytes;
  }
  repaintContents(false);
}

void ScopeScreen::singleShot() {

  timeScaleLastTrigger = timeScale;
  resizeContents(viewport()->width(), viewport()->height());
  refreshScope();
}

QSize ScopeScreen::sizeHint() const {

  return QSize(MINIMUM_WIDTH, MINIMUM_HEIGHT); 
}

QSizePolicy ScopeScreen::sizePolicy() const {

  return QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
}

modeType ScopeScreen::setMode(modeType p_mode) {

  mode = p_mode;
  repaintContents(false);
  return(mode);
}

edgeType ScopeScreen::setEdge(edgeType p_edge) {

  edge = p_edge;
  return(edge);
}

triggerModeType ScopeScreen::setTriggerMode(triggerModeType p_triggerMode) {

  triggerMode = p_triggerMode;

  if ((triggerMode == TRIGGERMODE_SINGLE) || (triggerMode == TRIGGERMODE_MIDI)) {  
    timeScaleLastTrigger = timeScale;
  }
  return(triggerMode);
}

int ScopeScreen::setCh1(int p_ch1) {

  ch1 = p_ch1;
  repaintContents(false);
  return(ch1);
}

int ScopeScreen::setCh2(int p_ch2) {       

  ch2 = p_ch2;  
  repaintContents(false);
  return(ch2); 
}              

double ScopeScreen::setZoom(double p_zoom) {

  zoom = p_zoom;
  repaintContents(false);
  return(zoom);
}

int ScopeScreen::setTriggerThrs(int p_triggerThrs) {

  triggerThrs = p_triggerThrs;
  return(triggerThrs);
}
 
int ScopeScreen::setTimeScale(int p_timeScale) {

  timeScale = p_timeScale;
  timeScaleBytes = adata->framesize * (int)((double)adata->rate * (double)timeScale / 1000.0);
  if ((triggerMode == TRIGGERMODE_SINGLE) || (triggerMode == TRIGGERMODE_MIDI)) {
    if (timeScaleLastTrigger > timeScale) {
      resizeContents((int)((double)viewport()->width() * (double)timeScaleLastTrigger / (double)timeScale), 
                     viewport()->height());
    } else {
      noScroll();
    }
  } 
  if (timeScaleBytes <= scopebufValidBytes) {
    repaintContents(false);
  }
  return(timeScale);
}

modeType ScopeScreen::getMode() {

  return(mode); 
}
 
edgeType ScopeScreen::getEdge() {

  return(edge); 
}
 
triggerModeType ScopeScreen::getTriggerMode() {

  return(triggerMode);
}
 
int ScopeScreen::getCh1() {

  return(ch1);
}
 
int ScopeScreen::getCh2() {

  return(ch2);
}             
              
double ScopeScreen::getZoom() {

  return(zoom); 
}

int ScopeScreen::getTriggerThrs() {

  return(triggerThrs);
}

int ScopeScreen::getTimeScale() {
 
  return(timeScale);
}

void ScopeScreen::viewportResizeEvent(QResizeEvent *ev)
{
  if ((triggerMode != TRIGGERMODE_SINGLE) && (triggerMode != TRIGGERMODE_MIDI)) {
    resizeContents(ev->size().width(), ev->size().height());
  }
  repaintContents(false);
}

void ScopeScreen::moveHandler(int, int) {

  dragFlag = true;
  timer->start(100, true);
}

void ScopeScreen::timerProc() {

  dragFlag = false;
  repaintContents(false);
}

void ScopeScreen::noScroll() {

  resizeContents(viewport()->width(), viewport()->height());
}
