#ifndef KSCOPESCREEN_H
#define KSCOPESCREEN_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qscrollview.h>
#include <qlabel.h>
#include <qsizepolicy.h>
#include <qsize.h>
#include <qevent.h>
#include "audiodata.h"

#define MINIMUM_WIDTH                 100
#define MINIMUM_HEIGHT                 50
#define TRIGGER_RANGE                4096

enum modeType { MODE_NORMAL, MODE_SUM, MODE_DIFF };
enum edgeType { EDGE_RISING, EDGE_FALLING };
enum triggerModeType { TRIGGERMODE_CONTINUOUS, TRIGGERMODE_TRIGGERED, TRIGGERMODE_SINGLE, TRIGGERMODE_MIDI };

class ScopeScreen : public QScrollView
{
  Q_OBJECT

  private:
    AudioData *adata;
    modeType mode;
    edgeType edge;
    triggerModeType triggerMode;
    int ch1, ch2;
    double zoom;
    int triggerThrs;
    bool triggered, dragFlag;
    int timeScale, timeScaleLastTrigger;
    int timeScaleBytes;
    unsigned char *scopebuf;
    int scopebufValidBytes;
    QTimer *timer;

  protected:
    virtual void viewportPaintEvent(QPaintEvent *);
    virtual void viewportResizeEvent (QResizeEvent *ev); 
    
  public:
    ScopeScreen(QWidget* parent=0, const char *name=0, AudioData *p_adata=0);
    ~ScopeScreen();
    virtual QSize sizeHint() const;
    virtual QSizePolicy sizePolicy() const;
    modeType setMode(modeType p_mode);
    edgeType setEdge(edgeType p_edge);
    triggerModeType setTriggerMode(triggerModeType p_triggerMode);
    int setCh1(int p_ch1);
    int setCh2(int p_ch2);
    double setZoom(double p_zoom);
    int setTriggerThrs(int p_triggerThrs);
    int setTimeScale(int p_timeScale);
    modeType getMode();
    edgeType getEdge();
    triggerModeType getTriggerMode();
    int getCh1();
    int getCh2();
    double getZoom();
    int getTriggerThrs();
    int getTimeScale();
    void noScroll();

  public slots: 
    void refreshScope();
    void singleShot();
    void moveHandler(int, int);
    void timerProc();
};
  
#endif
      
