#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qhbuttongroup.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include "audiodata.h"
#include "spectrum.h"
#include "spectrumscreen.h"

Spectrum::Spectrum(QWidget *parent, const char *name, AudioData *p_adata) : QVBox(parent, name) {

  adata = p_adata;
  spectrumScreen = new SpectrumScreen(this, "SpectrumScreen", adata);
  timer = new QTimer(this);
  QObject::connect(timer, SIGNAL(timeout()),
                   this, SLOT(timerProc()));
  initControls();
}

Spectrum::~Spectrum()
{
}

void Spectrum::refreshSpectrum() {

  spectrumScreen->refreshSpectrum();
}

void Spectrum::initControls() {

  int l1;

  QHBox *spectrumScaleBox = new QHBox(this);
  f_minLabel = new QLabel(spectrumScaleBox);
  f_minLabel->setText(QString::number(spectrumScreen->get_f_min())+" Hz");
  QWidget *dummy0 = new QWidget(spectrumScaleBox);
  dummy0->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  fLabel = new QLabel(spectrumScaleBox);
  fLabel->setText(QString::number(spectrumScreen->get_f())+" Hz");
  QWidget *dummy0a = new QWidget(spectrumScaleBox);
  dummy0a->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  f_maxLabel = new QLabel(spectrumScaleBox);
  f_maxLabel->setText(QString::number(spectrumScreen->get_f_max())+" Hz");
  spectrumScreen->setMouseTracking(true);
  QObject::connect(spectrumScreen, SIGNAL(f_minmaxChanged()),
                   this, SLOT(updateFreqMinMaxLabels()));
  QObject::connect(spectrumScreen, SIGNAL(freqChanged()),
                   this, SLOT(updateFreqLabel()));
  QHBox *spectrumControls = new QHBox(this);
  spectrumControls->setSpacing(10);
  spectrumControls->setMargin(10);
  QHBox *spectrumSlider = new QHBox(this);
  spectrumSlider->setSpacing(10);    
  spectrumSlider->setMargin(10);
  QHBox *viewGroupBox = new QHBox(this);
  viewGroupBox->setMargin(10);
  QGroupBox *viewGroup = new QGroupBox(3, Qt::Horizontal, "View", this);
  QWidget *dummy1 = new QWidget(spectrumControls);
  QHBox *ch1SelectorBox = new QHBox(spectrumControls);
  QWidget *dummy2 = new QWidget(spectrumControls);
  QHBox *ch2SelectorBox = new QHBox(spectrumControls);
  QWidget *dummy3 = new QWidget(spectrumControls);
  dummy1->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  dummy2->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  dummy3->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  ch1SelectorBox->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
  ch2SelectorBox->setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Minimum));
  QLabel *ch1Label = new QLabel(ch1SelectorBox);
  ch1Label->setText("Ch1: ");
  ch1Selector = new QComboBox(ch1SelectorBox);
  QLabel *ch2Label = new QLabel(ch2SelectorBox);       
  ch2Label->setText("Ch2: ");     
  ch2Selector = new QComboBox(ch2SelectorBox);
    ch1Selector->insertItem("None");
    ch2Selector->insertItem("None");
  for (l1 = 0; l1 < adata->channels; l1++) {
    ch1Selector->insertItem(QString::number(l1));
    ch2Selector->insertItem(QString::number(l1));
  }
  ch1Selector->setCurrentItem(1);
  ch2Selector->setCurrentItem(2);
  QHButtonGroup *radioButtons = new QHButtonGroup(spectrumControls);
  radioButtons->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QWidget *dummy4 = new QWidget(spectrumControls);
  dummy4->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  normButton = new QRadioButton("Normal   ", radioButtons);
  sumButton = new QRadioButton("Ch1+Ch2   ", radioButtons);
  radioButtons->setButton(spectrumScreen->getMode());
  QHBox *windowSelectorBox = new QHBox(spectrumControls);
  windowSelectorBox->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum)); 
  QLabel *windowLabel = new QLabel(windowSelectorBox);
  windowLabel->setText("Window: ");
  QWidget *dummy4a = new QWidget(spectrumControls);
  dummy4a->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  windowSelector = new QComboBox(windowSelectorBox);
  windowSelector->insertItem("Rectangle");
  windowSelector->insertItem("Bartlett");
  windowSelector->insertItem("Hann");
  windowSelector->insertItem("Welch");
  windowSelector->setCurrentItem((int)spectrumScreen->getWindow());
  QObject::connect(radioButtons, SIGNAL(clicked(int)),
                   this, SLOT(modeChanged(int)));
  QObject::connect(ch1Selector, SIGNAL(highlighted(int)),
                   this, SLOT(ch1Changed(int)));
  QObject::connect(ch2Selector, SIGNAL(highlighted(int)),
                   this, SLOT(ch2Changed(int))); 
  QObject::connect(windowSelector, SIGNAL(highlighted(int)),
                   this, SLOT(windowChanged(int)));

  QVBox *framesSliderBox = new QVBox(spectrumSlider);
  QHBox *framesLabelBox = new QHBox(framesSliderBox);
  QLabel *framesMinLabel = new QLabel(framesLabelBox);
  framesMinLabel->setText(QString::number(1<<MIN_FRAMES_LOG_2));
  QWidget *dummy5 = new QWidget(framesLabelBox);
  dummy5->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *framesLabel = new QLabel(framesLabelBox);
  framesLabel->setText("Frames");
  QWidget *dummy6 = new QWidget(framesLabelBox);
  dummy6->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *framesMaxLabel = new QLabel(framesLabelBox);
  framesMaxLabel->setText(QString::number(1<<MAX_FRAMES_LOG_2));
  framesSlider = new QSlider(MIN_FRAMES_LOG_2, MAX_FRAMES_LOG_2, 1, 
                             10, QSlider::Horizontal, framesSliderBox, "Frames");
  framesSlider->setTickInterval(1);
  framesSlider->setTickmarks(QSlider::Below);

  QLabel *dummyLabel1 = new QLabel(spectrumSlider);
  dummyLabel1->setText(" ");

  QVBox *zoomSliderBox = new QVBox(spectrumSlider);
  QHBox *zoomLabelBox = new QHBox(zoomSliderBox);  
  QLabel *zoomMinLabel = new QLabel(zoomLabelBox); 
  zoomMinLabel->setText(QString::number((double)SPECTRUM_MIN_ZOOM / 10.0));
  QWidget *dummy7 = new QWidget(zoomLabelBox);
  dummy7->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *zoomLabel = new QLabel(zoomLabelBox);
  zoomLabel->setText("Gain");
  QWidget *dummy8 = new QWidget(zoomLabelBox);
  dummy8->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum));
  QLabel *zoomMaxLabel = new QLabel(zoomLabelBox); 
  zoomMaxLabel->setText(QString::number((double)SPECTRUM_MAX_ZOOM / 10.0));
  zoomSlider = new QSlider(SPECTRUM_MIN_ZOOM, SPECTRUM_MAX_ZOOM, int(SPECTRUM_MAX_ZOOM / 10.0), 
                                int((double)spectrumScreen->getZoom() * 10.0),
                                QSlider::Horizontal, zoomSliderBox, "Gain");          
  zoomSlider->setTickInterval(int(SPECTRUM_MAX_ZOOM / 10.0));                    
  zoomSlider->setTickmarks(QSlider::Below);         

  QPushButton *zoom1Button = new QPushButton("Gain 1", spectrumSlider);
  QObject::connect(zoom1Button, SIGNAL(clicked()),
                   this, SLOT(setZoom1()));

  QHButtonGroup *fftModeButtons = new QHButtonGroup("Spectrum Mode", viewGroup);
  powButton = new QRadioButton("Pow ", fftModeButtons);
  absButton = new QRadioButton("Abs ", fftModeButtons);
  logButton = new QRadioButton("Log", fftModeButtons);
  fftModeButtons->setButton(spectrumScreen->getFFTMode());

  QHButtonGroup *viewModeButtons = new QHButtonGroup("Display Mode", viewGroup);
  fifButton = new QRadioButton("Spectrum ", viewModeButtons);
  fiftButton = new QRadioButton("Spectrum over time", viewModeButtons);
  viewModeButtons->setButton(spectrumScreen->getViewMode());

  QHButtonGroup *normModeButtons = new QHButtonGroup("Normalization", viewGroup);
  eachLineButton = new QRadioButton("Each Line ", normModeButtons);
  globalMaxButton = new QRadioButton("Global ", normModeButtons);
  fixedMaxButton = new QRadioButton("Fixed", normModeButtons);
  normModeButtons->setButton(spectrumScreen->getNormMode());

  QObject::connect(fftModeButtons, SIGNAL(clicked(int)),
                   this, SLOT(fftModeChanged(int)));
  QObject::connect(viewModeButtons, SIGNAL(clicked(int)),
                   this, SLOT(viewModeChanged(int)));
  QObject::connect(normModeButtons, SIGNAL(clicked(int)),
                   this, SLOT(normModeChanged(int)));
  QObject::connect(framesSlider, SIGNAL(valueChanged(int)),
                   this, SLOT(framesChanged(int)));
  QObject::connect(zoomSlider, SIGNAL(valueChanged(int)),
                   this, SLOT(zoomChanged(int)));
}

void Spectrum::modeChanged(int id) {

  switch(id) {
  case 0: spectrumScreen->setMode(SPECTRUM_MODE_NORMAL);
          break;
  case 1: spectrumScreen->setMode(SPECTRUM_MODE_SUM);
          break;
  }  
}

void Spectrum::triggerModeChanged(int id) {

  if ((spectrumScreen->getTriggerMode() == SPECTRUM_TRIGGERMODE_SINGLE)
      && (id == SPECTRUM_TRIGGERMODE_CONTINUOUS)) {
    timer->start(int((double)spectrumScreen->getFFTFrames() / (double)adata->rate * 1000.0), true);
  } 
  switch(id) {
  case 0: spectrumScreen->setTriggerMode(SPECTRUM_TRIGGERMODE_CONTINUOUS);
          break;
  case 1: spectrumScreen->setTriggerMode(SPECTRUM_TRIGGERMODE_SINGLE);
          break;
  }  
}

void Spectrum::fftModeChanged(int id) {

  switch(id) {
  case 0: spectrumScreen->setFFTMode(FFT_MODE_POW);
          break;
  case 1: spectrumScreen->setFFTMode(FFT_MODE_ABS);
          break;
  case 2: spectrumScreen->setFFTMode(FFT_MODE_LOG);
          break;
  }  
}

void Spectrum::viewModeChanged(int id) {

  switch(id) {
  case 0: spectrumScreen->setViewMode(VIEW_MODE_FIF);
          break;
  case 1: spectrumScreen->setViewMode(VIEW_MODE_FIFT);
          break;
  }  
}

void Spectrum::normModeChanged(int id) {
 
  switch(id) {
  case 0: spectrumScreen->setNormMode(NORM_MODE_EACH);
          break;
  case 1: spectrumScreen->setNormMode(NORM_MODE_GLOBAL);
          break;
  case 2: spectrumScreen->setNormMode(NORM_MODE_FIXED);
          break;
  }  
}

void Spectrum::ch1Changed(int index) {

  spectrumScreen->setCh1(index - 1);
}

void Spectrum::ch2Changed(int index) { 

  spectrumScreen->setCh2(index - 1);  
}

void Spectrum::windowChanged(int index) {

  switch(index) {
  case 0: spectrumScreen->setWindow(WINDOW_RECT);
          break;
  case 1: spectrumScreen->setWindow(WINDOW_BARTLETT);
          break;
  case 2: spectrumScreen->setWindow(WINDOW_HANN);
          break;
  case 3: spectrumScreen->setWindow(WINDOW_WELCH);
          break;
  }
}

void Spectrum::framesChanged(int val) { 

  spectrumScreen->setFFTFrames(1<<val);
}

void Spectrum::zoomChanged(int val) {
     
  spectrumScreen->setZoom((double)val / 10.0);
}

void Spectrum::updateFreqLabel() {

  fLabel->setText(QString::number(spectrumScreen->get_f())+" Hz");
}

void Spectrum::updateFreqMinMaxLabels() { 

  f_minLabel->setText(QString::number(spectrumScreen->get_f_min())+" Hz");
  f_maxLabel->setText(QString::number(spectrumScreen->get_f_max())+" Hz");
}

void Spectrum::timerProc() {                           

  if (spectrumScreen->getTriggerMode() != SPECTRUM_TRIGGERMODE_SINGLE) {
    timer->start(int((double)spectrumScreen->getFFTFrames() / (double)adata->rate * 1000.0), true);
  }
  refreshSpectrum();
}

void Spectrum::startSpectrum(bool run) {

  timerProc();
}

void Spectrum::setZoom1() {

  zoomSlider->setValue(int((double)spectrumScreen->setZoom(1.0) * 10.0));
}
