#ifndef KSPECTRUM_H
#define KSPECTRUM_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qwidget.h>
#include <qstring.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qcombobox.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include "spectrumscreen.h"

#define SPECTRUM_BORDER                      5
#define MIN_FRAMES_LOG_2                     6
#define MAX_FRAMES_LOG_2                    16
#define SPECTRUM_MIN_ZOOM                    5
#define SPECTRUM_MAX_ZOOM                  100

class Spectrum : public QVBox
{
  Q_OBJECT

  private:
    AudioData *adata;
    QComboBox *ch1Selector;
    QComboBox *ch2Selector;
    QComboBox *windowSelector;
    QRadioButton *sumButton, *diffButton, *normButton;
    QRadioButton *continuousButton, *triggeredButton, *singleButton;
    QRadioButton *powButton, *absButton, *logButton;
    QRadioButton *fifButton, *fiftButton;
    QRadioButton *eachLineButton, *globalMaxButton, *fixedMaxButton;
    QPushButton *triggerButton;
    QLabel *f_minLabel, *f_maxLabel, *fLabel;
    QSlider *framesSlider, *zoomSlider;
    QTimer *timer;

  public:
    SpectrumScreen *spectrumScreen;

  private:
    void initControls();

  public:
    Spectrum(QWidget* parent=0, const char *name=0, AudioData *p_adata=0);
    ~Spectrum();
    void startSpectrum(bool run);

  public slots:
    void modeChanged(int id);
    void triggerModeChanged(int id);
    void fftModeChanged(int id);
    void viewModeChanged(int id);
    void normModeChanged(int id);
    void ch1Changed(int index);
    void ch2Changed(int index);
    void windowChanged(int index);
    void framesChanged(int val);
    void zoomChanged(int val);
    void setZoom1();
    void updateFreqMinMaxLabels();
    void updateFreqLabel();
    void refreshSpectrum();
    void timerProc();
};

#endif

